/*
 * Copyright (c) Des Herriott 1993, 1994
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Des Herriott
 */

/*
 * Source file edops.c - operation prefixed by 0xED.
 */

/* in_b__c_ */
case 0x40:
	*b = in_byte(*bc);
	break;

/* out__c__b */
case 0x41:
	out_byte(*bc, *b);
	break;

/* sbc_hl_bc */
case 0x42:
	*hl = subtract_and_test16(*hl, *bc, Tst(carryFlag) ? 1 : 0);
	break;

/* ld (NN), bc */
case 0x43:
	work16 = GetNext2Ops;
	write_word(work16,*bc);
	break;

/* WARNING: neg - half-carry not implemented */
/* neg */
case 0x44:
	*a = subtract_and_test(0, *a, 0);
	break;

/* retn */
case 0x45:
	*pc = PopValue;
	theProcessor.iff1 = theProcessor.iff2;
	break;

/* im_0 */
case 0x46:
	theProcessor.im = 0;
	break;

/* ld_i_a */
case 0x47:
	theProcessor.int_vec = *a;
	break;

/* in_c__c_ */
case 0x48:
	*c = in_byte(*bc);
	break;

/* out__c__c */
case 0x49:
	out_byte(*bc, *c);
	break;

/* adc_hl_bc */
case 0x4A:
	*hl = add_and_test16(*hl, *bc, Tst(carryFlag)? 1 : 0);
	break;

/* ld bc,(NN) */
case 0x4B:
	work16 = GetNext2Ops;
	*bc = read_word(work16);
	break;

/* for our purposes, this is equivalent to ret */
/* reti */
case 0x4D:
	*pc = PopValue;
	break;

/* ld_r_a */
case 0x4F:
	theProcessor.refresh = *a;
	break;

/* in_d__c_ */
case 0x50:
	*d = in_byte(*bc);
	break;

/* out__c__d */
case 0x51:
	out_byte(*bc, *d);
	break;

/* sbc_hl_de */
case 0x52:
	*hl = subtract_and_test16(*hl, *de, Tst(carryFlag) ? 1 : 0);
	break;

/* ld__NN__de */
case 0x53:
	work16 = GetNext2Ops;
	write_word(work16, *de);
	break;

/* im_1 */
case 0x56:
	theProcessor.im = 1;
	break;

/* ld_a_i */
case 0x57:
	*a = theProcessor.int_vec;
	par_overFlag = theProcessor.iff2;
	signFlag = (*a & 0x80);
	zeroFlag = !(*a);
	break;

/* in_e__c_ */
case 0x58:
	*e = in_byte(*bc);
	break;

/* out__c__e */
case 0x59:
	out_byte(*bc, *e);
	break;

/* adc_hl_de */
case 0x5A:
	*hl = add_and_test16(*hl, *de, Tst(carryFlag)? 1 : 0);
	break;

/* ld_de__NN_ */
case 0x5B:
	work16 = GetNext2Ops;
	*de = read_word(work16);
	break;

/* im_2 */
case 0x5E:
	theProcessor.im = 2;
	break;

/* ld_a_r */
case 0x5F:
	theProcessor.refresh = instrs & 0x7f;
	*a = theProcessor.refresh;
	par_overFlag = theProcessor.iff2;
	signFlag = (*a & 0x80);
	zeroFlag = !(*a);
	break;

/* in_h__c_ */
case 0x60:
	*h = in_byte(*bc);
	break;

/* out__c__h */
case 0x61:
	out_byte(*bc, *h);
	break;

/* sbc_hl_hl */
case 0x62:
	*hl = subtract_and_test16(*hl, *hl, Tst(carryFlag) ? 1 : 0);
	break;

/* ld__nn__hl */
case 0x63:
	work16 = GetNext2Ops;
	write_word(work16, *hl);
	break;

/* rrd */
case 0x67:
	work8 = read_byte(*hl);
	work8_2 = (work8 >> 4) | ((*a & 0x0f) << 4);
	*a = (*a & 0xf0) | (work8 & 0x0f);
	signFlag = *a & 0x80;
	zeroFlag = !(*a);
	par_overFlag = parity_tbl[*a];
	write_byte(*hl,work8_2);
	break;

/* in_l__c_ */
case 0x68:
	*l = in_byte(*bc);
	break;

/* out__c__l */
case 0x69:
	out_byte(*bc, *l);
	break;

/* adc_hl_hl */
case 0x6A:
	*hl = add_and_test16(*hl, *hl, Tst(carryFlag)? 1 : 0);
	break;

/* ld_hl__NN_ also defined in z80ops.c */
case 0x6B:
	work16 = GetNext2Ops;
	*hl = read_word(work16);
	break;

/* rld */
case 0x6F:
	work8 = read_byte(*hl);
	work8_2 = ((work8 <<4) | (*a & 0x0f)) & 0xff;
	*a = (*a & 0xf0) | (work8 >> 4);
	signFlag = *a & 0x80;
	zeroFlag = !(*a);
	par_overFlag = parity_tbl[*a];
	write_byte(*hl, work8_2);
	break;

/* in_f__c_ */
case 0x70:
	*f = in_byte(*bc);
	RetrieveFlags();
	break;

/* sbc_hl_sp */
case 0x72:
	*hl = subtract_and_test16(*hl, *sp, Tst(carryFlag) ? 1 : 0);
	break;

/* ld__NN__sp */
case 0x73:
	work16 = GetNext2Ops;
	write_word(work16, *sp);
	break;

/* in_a__c_ */
case 0x78:
	*a = in_byte(*bc);
	break;

/* out__c__a */
case 0x79:
	out_byte(*bc, *a);
	break;

/* adc_hl_sp */
case 0x7A:
	*hl = add_and_test16(*hl, *sp, Tst(carryFlag)? 1 : 0);
	break;

/* ld_sp__NN_ */
case 0x7B:
	work16 = GetNext2Ops;
	*sp = read_byte(work16) + (read_byte(work16 + 1) << 8);
	break;

/* ldi */
case 0xA0:
	write_byte(*de,read_byte(*hl));
	(*de)++; (*hl)++; (*bc)--;
	par_overFlag = *bc; Clr(hcarryFlag); Clr(addsubFlag);
	break;

/* cpi */
case 0xA1:
	compare_and_test(read_byte(*hl));
	(*hl)++; (*bc)--;
	par_overFlag = *bc;
	break;

/* ini */
case 0xA2:
	write_byte(*hl, in_byte(*bc));
	(*b)--; (*hl)++;
	zeroFlag = !(*b);
	Set(addsubFlag);
	break;

/* outi */
case 0xA3:
	out_byte(*bc, read_byte(*hl));
	(*b)--; (*hl)++;
	zeroFlag = !(*b);
	Set(addsubFlag);
	break;

/* ldd */
case 0xA8:
	write_byte(*de,read_byte(*hl));
	(*de)--; (*hl)--; (*bc)--;
	par_overFlag = *bc; Clr(hcarryFlag); Clr(addsubFlag);
	break;

/* cpd */
case 0xA9:
	compare_and_test(read_byte(*hl));
	(*hl)--; (*bc)--;
	par_overFlag = *bc;
	break;

/* ind */
case 0xAA:
	write_byte(*hl, in_byte(*bc));
	(*b)--; (*hl)--;
	zeroFlag = !(*b);
	Set(addsubFlag);
	break;

/* outd */
case 0xAB:
	out_byte(*bc, read_byte(*hl));
	(*b)--; (*hl)--;
	zeroFlag = !(*b);
	Set(addsubFlag);
	break;

/* ldir */
case 0xB0:
	do {
		write_byte(*de,read_byte(*hl));
		(*hl)++; (*de)++; (*bc)--;
	} while (*bc);

	Clr(hcarryFlag); Clr(par_overFlag); Clr(addsubFlag);
	break;

/* cpir */
case 0xB1:
	do {
		compare_and_test(read_byte(*hl));
		(*hl)++; (*bc)--;
		par_overFlag = *bc;
	} while (*bc && !zeroFlag);
	break;

/* inir */
case 0xB2:
	do {
		write_byte(*hl, in_byte(*bc));
		(*b)--; (*hl)++;
	} while (*b);

	Set(zeroFlag); Set (addsubFlag);
	break;

/* otir */
case 0xB3:
	do {
		out_byte(*bc, read_byte(*hl));
		(*b)--; (*hl)++;
	} while (*b);

	Set(zeroFlag); Set(addsubFlag);
	break;

/* lddr */
case 0xB8:
	do {
		write_byte(*de,read_byte(*hl));
		(*hl)--; (*de)--; (*bc)--;
	} while (*bc);

	Clr(hcarryFlag); Clr(par_overFlag); Clr(addsubFlag);
	break;

/* cpdr */
case 0xB9:
	do {
		compare_and_test(read_byte(*hl));
		(*hl)--; (*bc)--;
		par_overFlag = *bc;
	} while (*bc && !zeroFlag);
	break;

/* indr */
case 0xBA:
	do {
		write_byte(*hl, in_byte(*bc));
		(*b)--; (*hl)--;
	} while (*b);

	Set(zeroFlag); Set (addsubFlag);
	break;

/* otdr */
case 0xBB:
	do {
		out_byte(*bc, read_byte(*hl));
		(*b)--; (*hl)--;
	} while (*b);

	Set(zeroFlag); Set(addsubFlag);
	break;


/*
 * Now some pseudo-ops with special meaning to xzx
 */

/* debug trap */
case 0x00:
	xzx_mesg(XZX_INFO, "ED 00 - processor trap");
	dump_cpu_state();
	fprintf(stderr, "  <q> to quit, any other key to continue: ");
	do {
		work8 = getchar();
		if (work8 == 'q' || work8 == 'Q') Quit(0);
	} while (work8 == 0xff);
	break;

#ifdef LEVEL_LOADER
/* Level loader trap */
case 0xFB:
	level_loader_trap();
	break;
#endif

#ifdef LOAD_SAVE
case 0xFC:
	loader();
	break;

case 0xFD:
	saver();
	break;
#endif

#ifdef XZX_IF1
/* Interface I RS232 I/O traps */
case 0xFE:
	rs232_ip_trap();
	break;

case 0xFF:
	rs232_op_trap();
	break;
#endif

/* Unimplemented */
default:
	xzx_mesg(XZX_ERR, "unimplemented instruction ED %02X",
		read_byte(*pc - 1));
	break;

