/*
 * Copyright (c) Des Herriott 1993, 1994
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Des Herriott
 */

/*
 * Source file mem.c - where memory is actually held.
 *
 * Memory is implemented as an array of 4 16K pages.  Virtual page
 * numbers range from 0..3, real page numbers range from 0..11.
 * <real_pageno> is used to decide if the page is read-only memory.
 */

#include <stdlib.h>
#include <stdio.h>

#include "z80.h"
#include "mem.h"
#include "util.h"
#include "rtcfg.h"

uns8 *theMemory[4];		/* 4 pointers to real memory */
int real_pagenos[4];	/* index to real memory page */
uns8 *realMem[NPAGES];	/* real memory is kept here */

/* If this is set, write_byte and write_word will overwrite ROM */
int rom_override = 0;


/* Allocate NPAGES chunks of memory, each of size PAGESIZE */
void
init_memory(void)
{
	static int inited = 0;
	int i;

	if (inited) {
		xzx_mesg(XZX_ERR, "attempt to re-initialise memory (failed)");
		return;
	}

	for (i = 0; i < NPAGES; i++) {
		if ((realMem[i] = (uns8 *)malloc(PAGESIZE)) == (uns8 *)0)
			xzx_mesg(XZX_FATAL, "can't allocate 16K for page %d!\n", i);
	}

}


/* Page in real memory page at <frame>, where <frame> is in range 0..3 */
void
page_in(int frame, int rpage)
{
	/* only the +3 allows RAM at all 4 frames */
	if (GetCfg(emode) != 3 && frame == 0 && rpage >= RAM0) {
		xzx_mesg(XZX_WARN, "attempt to page RAM in at 0x0000 (ignored)");
		return;
	}

	theMemory[frame] = realMem[rpage];
	real_pagenos[frame] = rpage;

}
