
/*
 * Copyright (c) Des Herriott 1993, 1994
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Des Herriott
 */

/*
 * Header file mem.h - contains definitions, macros and routines 
 * for accessing the segmented memory model.
 */

#ifndef _XZX_MEM_H
#define _XZX_MEM_H

#include "xspec.h"
#include "rtcfg.h"

#if !defined(__GNUC__)
#define inline
#endif

extern int rom_override;

/* 16K memory pages */
#define PAGESIZE 16384

enum {
	ROM0, ROM1, ROM2, ROM3,
	RAM0, RAM1, RAM2, RAM3, RAM4, RAM5, RAM6, RAM7, 
	NPAGES
};


extern uns8 *theMemory[4];
extern int real_pagenos[4];
extern uns8 *realMem[NPAGES];

/* Map a 16 bit address through the memory indirection - allows fast paging */
#define VPAGE(addr) ((addr) >> 14)
#define RPAGE(addr) (real_pagenos[VPAGE(addr)])
#define MEM(addr) (theMemory[VPAGE(addr)][(addr) & 0x3fff])
#define READ_ONLY(addr) (RPAGE(addr) < RAM0)

void init_memory(void);
void page_in(int frame, int rpage);


/*
 * I know putting blocks of code in header files isn't very nice, but
 * this does let me take advantage of gcc's inlining...
 */
static inline uns8 
read_byte(uns16 addr)
{
	return MEM(addr);
}


static inline uns16
read_word(uns16 addr)
{
	return ((MEM(addr)) + (MEM((uns16)(addr + 1)) << 8));
}


static inline void 
write_byte(uns16 addr, uns8 val)
{
	/* both from xspec.c */
	extern void (*screen_write)(uns16, uns8);
	extern int screen_select;

	if (READ_ONLY(addr) && !rom_override) return;

	MEM(addr) = val;

	if (RPAGE(addr) == screen_select) {
		if ((addr & 0x3fff) < 6144)
			screen_write(addr, val);
		else if ((addr & 0x3fff) < 6912)
			attr_write(addr, val);
	}
}


static inline void
write_word(uns16 addr, uns16 val)
{
	write_byte(addr, val & 0xff);
	write_byte((uns16)(addr + 1), (val & 0xff00) >> 8);
}


#endif
