
/*
 * Copyright (c) Des Herriott 1993, 1994
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Des Herriott
 */

/* Header file rtcfg.h - for reading & setting run-time configuration */

#ifndef _XZX_RTCFG_H
#define _XZX_RTCFG_H

#include "string.h"

#define DEF_EMODE 48
#define DEF_FONT "fixed"

struct cfg_block {
	int emode;			/* what kind of Speccy to emulate */
	char geometry[256];	/* X11 geometry spec. (size is ignored) */
	char font[256];		/* X11 font name */
	char visual[16];	/* X11 Visual name */
	int use_mitshm;		/* attempt to use MIT-SHM for image transfer */
	int pixmaps;		/* attempt to use pixmaps for image transfer */
	int priv_cmap;		/* use a private colormap */
	int monochrome;		/* use only BlackPixel and WhitePixel */
	int recalibrate;	/* attempt to keep the interrupt rate at 50/sec */
	char rom48[256];	/* spectrum ROM image filename */
	char rom128_0[256];	/* 128K ROM 0 */
	char rom128_1[256];	/* 128K ROM 1 */
	char rompl3_0[256];	/* +3 ROM 0 */
	char rompl3_1[256];	/* +3 ROM 1 */
	char rompl3_2[256];	/* +3 ROM 2 */
	char rompl3_3[256];	/* +3 ROM 3 */
	char lib_dir[1024];	/* directory to check for support files */
	char disk_a[256];	/* +3 floppy disk A: */
	char disk_b[256];	/* +3 floppy disk B: */
	int icnt;			/* whether or not to use counter-style interrupts */
	int ifreq;			/* how often interrupts arrive (counter-style only) */
	int flashing;		/* is flashing enabled? */
	int rr_shm;			/* refresh frequency when using MITSHM */
	int rr_noshm;		/* as above, but not using MITSHM */
#ifdef XZX_IF1
	int if1_active;		/* are we emulating if1 ? */
	char if1_rom[256];	/* interface 1 ROM image filename */
	char cartfiles[8][256]; /* names of cartridge files */	
	int translate_nl;	/* Translate CR <-> LF on RS232 input? */
	int strip_nl;		/* Strip CR on RS232 output? */
#endif
#ifdef SLOWDOWN
	int slowdown;		/* slowdown factor for very fast machines */
#endif
#ifdef JOY
	int use_js;			/* try to use a joystick */
	int js_tolerance;	/* joystick displacement tolerance */
	char js_device[256];/* joystick device name (/dev/js[0|1]) */
#endif
#if defined(DEV_AUDIO) || defined(PCSPKR_AUDIO) || defined(NAS_AUDIO)
	int use_sound;		/* use Sparc, PC or NAS audio to emulate the speaker */
#ifdef DEV_AUDIO
	char audio_dev[256];/* name of audio device */
#endif
#ifdef NAS_AUDIO
	char au_serv[256];	/* name of audio server */
#endif
#endif
};

extern struct cfg_block cblock;

#define GetCfg(field) cblock.field
#define SetCfg(field,value) cblock.field = value
#define SetCfgStr(field,value) strcpy(cblock.field,value)

#endif

