
/*
 * Copyright (c) Des Herriott 1993, 1994
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Des Herriott
 */

/* Header file xspecP.h - some definitions and tables used only by xspec.c */

#if SCALING>1
/* This is for halftoning if we're working in monochrome and scaling is in
 * operation */
#define shade(val,inkpap,paper)  (((val)&(inkpap))^(paper))
static int scaleup_done=0;       /* initialisation done or not? 0=no */
static long scaleup[256];   /* to hold table of scaled up bytes, e.g.
			                   00110100B -> 0000111100110000B */
static uns8 halftone[16][4]={    /* 8-by-4 bitmaps for halftoning */
	{0xff,0xff,0xff,0xff},   /* black */
	{0xee,0xff,0xbb,0xff},   /* blue */
	{0x55,0xff,0xaa,0xff},   /* red */
	{0x55,0xbb,0x55,0xff},   /* magenta */
	{0xaa,0x55,0xaa,0x55},   /* green */
	{0x51,0xaa,0x15,0xaa},   /* cyan */
	{0xaa,0x44,0xaa,0x00},   /* yellow */
	{0x11,0x00,0x44,0x00},   /* white */
	{0xff,0xff,0xff,0xff},   /* black */
	{0x77,0xdf,0x77,0xfd},   /* bright blue */
	{0xaa,0xdd,0xaa,0x77},   /* bright red */
	{0xae,0x55,0xea,0x55},   /* bright magenta */
	{0x55,0x22,0x55,0x88},   /* bright green */
	{0x88,0x22,0x88,0x22},   /* bright cyan */
	{0x00,0x40,0x00,0x04},   /* bright yellow */
	{0x00,0x00,0x00,0x00}    /* bright white */
};
#endif

/* Maps ASCII keys to a keyboard row and bit to set/reset */
/* E.g. ktbl['a'] means set/reset bit 0 (0xfe) of keyboard row 1 */
struct ktbl {
	int port;	/* which keyboard row is the key on? */
	int bit;	/* which bit to mask */
} keytable[128] = {
	{8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, 
	{8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, 
	{8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, 
	{8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, 
	/* SPACE ! "" # $ % & ' */
	{7, 0xfe}, {3, 0xfe}, {3, 0xfd}, {3, 0xfb}, {3,0xf7},
	{3,0xef}, {4,0xf7}, {8,0},
	/* ( ) * + , - . / */
	{4, 0xfd}, {4, 0xfe}, {4, 0xfb}, {8, 0}, {8, 0}, {8, 0},
	{8, 0}, {8, 0}, 
	/* 0 1 2 3 4 5 6 7 8 9 */
	{4, 0xfe}, {3, 0xfe}, {3, 0xfd}, {3, 0xfb}, {3, 0xf7},
	{3, 0xef}, {4, 0xef}, {4, 0xf7}, {4, 0xfb}, {4, 0xfd},
	/* : ; < = > ? @ */
	{8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}, {3, 0xfd}, 
	/* A -> Z  (upper & lower case are the same) */
	{1, 0xfe}, {7, 0xef}, {0, 0xf7}, {1, 0xfb}, {2, 0xfb}, {1, 0xf7},
	{1, 0xef}, {6, 0xef}, {5, 0xfb}, {6, 0xf7}, {6, 0xfb}, {6, 0xfd}, 
	{7, 0xfb}, {7, 0xf7}, {5, 0xfd}, {5, 0xfe}, {2, 0xfe}, {2, 0xf7},
	{1, 0xfd}, {2, 0xef}, {5, 0xf7}, {0, 0xef}, {2, 0xfd}, {0, 0xfb},
	{5, 0xef}, {0, 0xfd},
	/* [ \ ] ^ _ ` */
	{8, 0}, {8, 0}, {8, 0}, {4, 0xef}, {8, 0}, {8, 0},
	/* a -> z  (upper & lower case are the same) */
	{1, 0xfe}, {7, 0xef}, {0, 0xf7}, {1, 0xfb}, {2, 0xfb}, {1, 0xf7},
	{1, 0xef}, {6, 0xef}, {5, 0xfb}, {6, 0xf7}, {6, 0xfb}, {6, 0xfd}, 
	{7, 0xfb}, {7, 0xf7}, {5, 0xfd}, {5, 0xfe}, {2, 0xfe}, {2, 0xf7},
	{1, 0xfd}, {2, 0xef}, {5, 0xf7}, {0, 0xef}, {2, 0xfd}, {0, 0xfb},
	{5, 0xef}, {0, 0xfd},
	/* { | } ~ DEL */
	{8, 0}, {8, 0}, {8, 0}, {8, 0}, {8, 0}
};


/* RGB triples describing the Spectrum's 16 colours.  Note that black
 * and bright black are the same.
 */
#define NCOLORS 16
static unsigned short rgb_vals[NCOLORS][3] = {
	/* dull */
	{ 0x0000, 0x0000, 0x0000}, { 0x0000, 0x0000, 0xbfff},
	{ 0xbfff, 0x0000, 0x0000}, { 0xbfff, 0x0000, 0xbfff},
	{ 0x0000, 0xbfff, 0x0000}, { 0x0000, 0xbfff, 0xbfff},
	{ 0xbfff, 0xbfff, 0x0000}, { 0xbfff, 0xbfff, 0xbfff}, 
	/* bright */
	{ 0x0000, 0x0000, 0x0000}, { 0x0000, 0x0000, 0xffff},
	{ 0xffff, 0x0000, 0x0000}, { 0xffff, 0x0000, 0xffff},
	{ 0x0000, 0xffff, 0x0000}, { 0x0000, 0xffff, 0xffff},
	{ 0xffff, 0xffff, 0x0000}, { 0xffff, 0xffff, 0xffff},
};


/* Definitions for 128K and +3 memory management */

#define PORT_BANKM 0x7ffd
#define PORT_BANK678 0x1ffd

#define RAM_MASK 0x07
#define ROM_MASK 0x10
#define SCREEN_MASK 0x08
#define PAGE_DISABLE 0x20

#define PAGING_DISABLED(port) ((port) & PAGE_DISABLE)
#define EXTENDED_RAM(port) ((port) & 0x01)

