/*
 * Copyright (c) Des Herriott 1993, 1994
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holder not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Des Herriott
 */

/* Source file z80ops.c - contains the regular Z80 operations */

/* nop */
case 0x00:
	break;

/* ld bc,NN */
case 0x01:
	*bc = GetNext2Ops;
	break;

/* ld (bc), a */
case 0x02:
	write_byte(*bc, *a);
	break;

/* inc bc */
case 0x03:
	(*bc)++;
	break;

/* inc b */
case 0x04:
	(*b)++;
	inc_test(*b);
	break;

/* dec b */
case 0x05:
	(*b)--;
	dec_test(*b);
	break;

/* ld b,N */
case 0x06:
	*b = GetNextOp;
	break;

/* */
case 0x07:
	if ((carryFlag = (*a & 0x80))) {
		*a = (*a << 1) | 0x01;
	} else
		*a <<= 1;
	Clr(hcarryFlag); Clr(addsubFlag);
	break;

/* ex af,af' */
case 0x08:
	StoreFlags();
	work16 = theProcessor.af_alt; theProcessor.af_alt = *af; *af = work16;
	RetrieveFlags();
	break;

/* */
case 0x09:
	work32 = *hl + *bc;
	carryFlag = work32 & 0x10000;
	Clr(addsubFlag);
	*hl = (uns16)work32;
	break;

/* ld a,(bc) */
case 0x0A:
	*a = read_byte(*bc);
	break;

/* dec bc */
case 0x0B:
	(*bc)--;
	break;

/* inc c */
case 0x0C:
	(*c)++;
	inc_test(*c);
	break;

/* dec c */
case 0x0D:
	(*c)--;
	dec_test(*c);
	break;

/* ld c,N */
case 0x0E:
	*c = GetNextOp;
	break;

/* rrca */
case 0x0F:
	if ((carryFlag = (*a & 0x01))) {
		*a = (*a >> 1) | 0x80;
	} else
		*a >>= 1;
	Clr(hcarryFlag); Clr(addsubFlag);
	break;

/* djnz D */
case 0x10:
	work8 = GetNextOp;  /* the offset */
	(*b)--;
	if (*b)
		*pc += (sgn8)work8;
	break;

/* ld de,NN */
case 0x11:
	*de = GetNext2Ops;
	break;

/* ld (de),a */
case 0x12:
	write_byte(*de,*a);
	break;

/* inc de */
case 0x13:
	(*de)++;
	break;

/* inc d */
case 0x14:
	(*d)++;
	inc_test(*d);
	break;

/* dec d */
case 0x15:
	(*d)--;
	dec_test(*d);
	break;

/* ld d,N */
case 0x16:
	*d = GetNextOp;
	break;

/* rla */
case 0x17:
	work8 = carryFlag ? 0x01 : 0;
	carryFlag = *a & 0x80;
	*a = (*a << 1) | work8;
	Clr(hcarryFlag); Clr(addsubFlag);
	break;

/* jr D */
case 0x18:
	*pc += (sgn8)GetNextOp;
	break;

/* add hl, de */
case 0x19:
	work32 = *hl + *de;
	carryFlag = work32 & 0x10000;
	Clr(addsubFlag);
	*hl = (uns16)work32;
	break;

/* ld a,(de) */
case 0x1A:
	*a = read_byte(*de);
	break;

/* dec de */
case 0x1B:
	(*de)--;
	break;

/* inc e */
case 0x1C:
	(*e)++;
	inc_test(*e);
	break;

/* dec e */
case 0x1D:
	(*e)--;
	dec_test(*e);
	break;

/* ld e,N */
case 0x1E:
	*e = GetNextOp;
	break;

/* rra */
case 0x1F:
	work8 = carryFlag ? 0x80 : 0;
	carryFlag = *a & 0x01;
	*a = (*a >> 1) | work8;
	Clr(hcarryFlag); Clr(addsubFlag);
	break;

/* jr nz,D */
case 0x20:
	work8 = GetNextOp;
	if (!Tst(zeroFlag))
		*pc += (sgn8)work8;
	break;

/* ld hl,NN */
case 0x21:
	*hl = GetNext2Ops;
	break;

/* ld__NN_hl */
case 0x22:
	work16 = GetNext2Ops;
	write_word(work16,*hl);
	break;

/* inc_hl */
case 0x23:
	(*hl)++;
	break;

/* inc_h */
case 0x24:
	(*h)++;
	inc_test(*h);
	break;

/* dec_h */
case 0x25:
	(*h)--;
	dec_test(*h);
	break;

/* ld_hN */
case 0x26:
	*h = GetNextOp;
	break;

/* This one's going to be fun :-) */
/* daa */
case 0x27:
	subflg = addsubFlag;

	high_nib = (*a & 0xf0) >> 4; low_nib  = *a &0x0f;

	if (!Tst(addsubFlag)) {
		if (!Tst(carryFlag)) {
			if (!Tst(hcarryFlag)) {
				if (low_nib < 10) {
					if (high_nib < 10) {
						add = 0x00; carry = 0;
					} else {
						add = 0x60; carry = 1;
					}
				} else {
					if (high_nib < 9) {
						add = 0x06; carry = 0;
					} else {
						add = 0x66; carry = 1;
					}
				}
			} else {	/* hcarryFlag */
				if (high_nib < 10) {
					add = 0x06; carry = 0;
				} else {
					add = 0x66; carry = 1;
				}
			}	/* hcarryFlag */
		} else {	/* carryFlag */
			if (!Tst(hcarryFlag)) {
				if (low_nib < 10) {
					add = 0x60; carry = 1;
				} else {
					add = 0x66; carry = 1;
				}
			} else {
				add = 0x66; carry = 1;
			}
		}	/* carryFlag */
	} else {	/* addsubFlag */
		if (!Tst(carryFlag)) {
			if (!Tst(hcarryFlag)) {
				add = 0x00; carry = 0;
			} else {
				add = 0xFA; carry = 0;
			}
		} else {
			if (!Tst(hcarryFlag)) {
				add = 0xA0; carry = 1;
			} else {
				add = 0x9A; carry = 1;
			}
		}
	}
	*a = add_and_test(*a, add, 0);
	addsubFlag = subflg;
	par_overFlag = parity_tbl[*a];
	carryFlag = carry;
	break;

/* jr_zD */
case 0x28:
	work8 = GetNextOp;
	if(Tst(zeroFlag))
		*pc += (sgn8)work8;
	break;

/* add hl,hl(): WARNING - half carry flag not implemented yet */
case 0x29:
	work32 = *hl << 1;
	carryFlag = work32 & 0x10000;
	Clr(addsubFlag);
	*hl = (uns16)work32;
	break;

/* ld hl,(NN) */
case 0x2A:
	work16 = GetNext2Ops;
	*hl = read_word(work16);
	break;

/* dec_hl */
case 0x2B:
	(*hl)--;
	break;

/* inc_l */
case 0x2C:
	(*l)++;
	inc_test(*l);
	break;

/* dec_l */
case 0x2D:
	(*l)--;
	dec_test(*l);
	break;

/* ld_lN */
case 0x2E:
	*l = GetNextOp;
	break;

/* cpl */
case 0x2F:
	*a ^= 0xff;
	Set(addsubFlag); Set(hcarryFlag);
	break;

/* jr_ncD */
case 0x30:
	work8 = GetNextOp;

	if(!Tst(carryFlag))
		*pc += (sgn8)work8;
	break;

/* ld_spNN */
case 0x31:
	*sp = GetNext2Ops;
	break;

/* ld__NN_a */
case 0x32:
	work16 = GetNext2Ops;
	write_byte(work16,*a);
	break;

/* inc_sp */
case 0x33:
	(*sp)++;
	break;

/* inc__hl_ */
case 0x34:
	write_byte(*hl, read_byte(*hl) + 1);
	inc_test(read_byte(*hl));
	break;

/* dec__hl_ */
case 0x35:
	write_byte(*hl, read_byte(*hl) - 1);
	dec_test(read_byte(*hl));
	break;

/* ld (hl),N */
case 0x36:
	write_byte(*hl,GetNextOp);
	break;

/* scf */
case 0x37:
	Set(carryFlag);
	Clr(hcarryFlag); Clr(addsubFlag);
	break;

/* jr_cD_ */
case 0x38:
	work8 = GetNextOp;

	if(Tst(carryFlag))
		*pc += (sgn8)work8;
	break;

/* add_hlsp */
case 0x39:
	work32 = *hl + *sp;
	carryFlag = work32 & 0x10000;
	Clr(addsubFlag);
	*hl = (uns16)work32;
	break;

/* ld a,(NN) */
case 0x3A:
	*a = read_byte(GetNext2Ops);
	break;

/* dec_sp */
case 0x3B:
	(*sp)--;
	break;

/* inc_a */
case 0x3C:
	(*a)++;
	inc_test(*a);
	break;

/* dec_a */
case 0x3D:
	(*a)--;
	dec_test(*a);
	break;

/* ld_aN */
case 0x3E:
	*a = GetNextOp;
	break;

/* ccf */
case 0x3F:
	Cpl(carryFlag);
	Clr(addsubFlag);
	/* half carry is set to a "don't care" value */
	break;

/* ld_bb */
case 0x40:
	/* pointless */
	break;

/* ld_bc */
case 0x41:
	*b = *c;
	break;

/* ld_bd */
case 0x42:
	*b = *d;
	break;

/* ld_be */
case 0x43:
	*b = *e;
	break;

/* ld_bh */
case 0x44:
	*b = *h;
	break;

/* ld_bl */
case 0x45:
	*b = *l;
	break;

/* ld_b_hl_ */
case 0x46:
	*b = read_byte(*hl);
	break;

/* ld_ba */
case 0x47:
	*b = *a;
	break;

/* ld_cb */
case 0x48:
	*c = *b;
	break;

/* ld_cc */
case 0x49:
	break;

/* ld_cd */
case 0x4A:
	*c = *d;
	break;

/* ld_ce */
case 0x4B:
	*c = *e;
	break;

/* ld_ch */
case 0x4C:
	*c = *h;
	break;

/* ld_cl */
case 0x4D:
	*c = *l;
	break;

/* ld_c_hl_ */
case 0x4E:
	*c = read_byte(*hl);
	break;

/* ld_ca */
case 0x4F:
	*c = *a;
	break;

/* ld_db */
case 0x50:
	*d = *b;
	break;

/* ld_dc */
case 0x51:
	*d = *c;
	break;

/* ld_dd */
case 0x52:
	break;

/* ld_de */
case 0x53:
	*d = *e;
	break;

/* ld_dh */
case 0x54:
	*d = *h;
	break;

/* ld_dl */
case 0x55:
	*d = *l;
	break;

/* ld_d_hl_ */
case 0x56:
	*d = read_byte(*hl);
	break;

/* ld_da */
case 0x57:
	*d = *a;
	break;

/* ld_eb */
case 0x58:
	*e = *b;
	break;

/* ld_ec */
case 0x59:
	*e = *c;
	break;

/* ld_ed */
case 0x5A:
	*e = *d;
	break;

/* ld_ee */
case 0x5B:
	break;

/* ld_eh */
case 0x5C:
	*e = *h;
	break;

/* ld_el */
case 0x5D:
	*e = *l;
	break;

/* ld_e_hl_ */
case 0x5E:
	*e = read_byte(*hl);
	break;

/* ld_ea */
case 0x5F:
	*e = *a;
	break;

/* ld_hb */
case 0x60:
	*h = *b;
	break;

/* ld_hc */
case 0x61:
	*h = *c;
	break;

/* ld_hd */
case 0x62:
	*h = *d;
	break;

/* ld_he */
case 0x63:
	*h = *e;
	break;

/* ld_hh */
case 0x64:
	break;

/* ld_hl */
case 0x65:
	*h = *l;
	break;

/* ld_h_hl_ */
case 0x66:
	*h = read_byte(*hl);
	break;

/* ld_ha */
case 0x67:
	*h = *a;
	break;

/* ld_lb */
case 0x68:
	*l = *b;
	break;

/* ld_lc */
case 0x69:
	*l = *c;
	break;

/* ld_ld */
case 0x6A:
	*l = *d;
	break;

/* ld_le */
case 0x6B:
	*l = *e;
	break;

/* ld_lh */
case 0x6C:
	*l = *h;
	break;

/* ld_ll */
case 0x6D:
	break;

/* ld_l_hl_ */
case 0x6E:
	*l = read_byte(*hl);
	break;

/* ld_la */
case 0x6F:
	*l = *a;
	break;

/* ld__hl_b */
case 0x70:
	write_byte(*hl,*b);
	break;

/* ld__hl_c */
case 0x71:
	write_byte(*hl,*c);
	break;

/* ld__hl_d */
case 0x72:
	write_byte(*hl,*d);
	break;

/* ld__hl_e */
case 0x73:
	write_byte(*hl,*e);
	break;

/* ld__hl_h */
case 0x74:
	write_byte(*hl,*h);
	break;

/* ld__hl_l */
case 0x75:
	write_byte(*hl,*l);
	break;

/* Stop until an interrupt is received */
/* halt */
case 0x76:
	(*pc)--;
	break;

/* ld__hl_a */
case 0x77:
	write_byte(*hl,*a);
	break;

/* ld_ab */
case 0x78:
	*a = *b;
	break;

/* ld_ac */
case 0x79:
	*a = *c;
	break;

/* ld_ad */
case 0x7A:
	*a = *d;
	break;

/* ld_ae */
case 0x7B:
	*a = *e;
	break;

/* ld_ah */
case 0x7C:
	*a = *h;
	break;

/* ld_al */
case 0x7D:
	*a = *l;
	break;

/* ld_a_hl_ */
case 0x7E:
	*a = read_byte(*hl);
	break;

/* ld_aa */
case 0x7F:
	break;


/* Now follow the adding routines.  All the flags work is done by a 
 * subsidiary function. */

/* add_ab */
case 0x80:
	*a = add_and_test(*a, *b, 0);
	break;

/* add_ac */
case 0x81:
	*a = add_and_test(*a, *c, 0);
	break;

/* add_ad */
case 0x82:
	*a = add_and_test(*a, *d, 0);
	break;

/* add_ae */
case 0x83:
	*a = add_and_test(*a, *e, 0);
	break;

/* add_ah */
case 0x84:
	*a = add_and_test(*a, *h, 0);
	break;

/* add_al */
case 0x85:
	*a = add_and_test(*a, *l, 0);
	break;

/* add_a_hl_ */
case 0x86:
	*a = add_and_test(*a, read_byte(*hl), 0);
	break;

/* add_aa */
case 0x87:
	*a = add_and_test(*a, *a, 0);
	break;

/* Add with carry... */
/* adc_ab */
case 0x88:
	*a = add_and_test(*a, *b, Tst(carryFlag)? 1 : 0);
	break;

/* adc_ac */
case 0x89:
	*a = add_and_test(*a, *c, Tst(carryFlag)? 1 : 0);
	break;

/* adc_ad */
case 0x8A:
	*a = add_and_test(*a, *d, Tst(carryFlag)? 1 : 0);
	break;

/* adc_ae */
case 0x8B:
	*a = add_and_test(*a, *e, Tst(carryFlag)? 1 : 0);
	break;

/* adc_ah */
case 0x8C:
	*a = add_and_test(*a, *h, Tst(carryFlag)? 1 : 0);
	break;

/* adc_al */
case 0x8D:
	*a = add_and_test(*a, *l, Tst(carryFlag)? 1 : 0);
	break;

/* adc_a_hl_ */
case 0x8E:
	*a = add_and_test(*a, read_byte(*hl), Tst(carryFlag)? 1 : 0);
	break;

/* adc_aa */
case 0x8F:
	*a = add_and_test(*a, *a, Tst(carryFlag)? 1 : 0);
	break;

/* And now for the subtractions... */

/* sub_b */
case 0x90:
	*a = subtract_and_test(*a, *b, 0);
	break;

/* sub_c */
case 0x91:
	*a = subtract_and_test(*a, *c, 0);
	break;

/* sub_d */
case 0x92:
	*a = subtract_and_test(*a, *d, 0);
	break;

/* sub_e */
case 0x93:
	*a = subtract_and_test(*a, *e, 0);
	break;

/* sub_h */
case 0x94:
	*a = subtract_and_test(*a, *h, 0);
	break;

/* sub_l */
case 0x95:
	*a = subtract_and_test(*a, *l, 0);
	break;

/* sub__hl_ */
case 0x96:
	*a = subtract_and_test(*a, read_byte(*hl), 0);
	break;

/* special case */
/* sub_a */
case 0x97:
	*a = 0;
	Set(zeroFlag); Clr(carryFlag); Clr(par_overFlag);
	Clr(signFlag); Set(addsubFlag);
	break;

/* subtract with carry... */
/* sbc_ab */
case 0x98:
	*a = subtract_and_test(*a, *b, Tst(carryFlag)? 1: 0);
	break;

/* sbc_ac */
case 0x99:
	*a = subtract_and_test(*a, *c, Tst(carryFlag)? 1: 0);
	break;

/* sbc_ad */
case 0x9A:
	*a = subtract_and_test(*a, *d, Tst(carryFlag)? 1: 0);
	break;

/* sbc_ae */
case 0x9B:
	*a = subtract_and_test(*a, *e, Tst(carryFlag)? 1: 0);
	break;

/* sbc_ah */
case 0x9C:
	*a = subtract_and_test(*a, *h, Tst(carryFlag)? 1: 0);
	break;

/* sbc_al */
case 0x9D:
	*a = subtract_and_test(*a, *l, Tst(carryFlag)? 1: 0);
	break;

/* sbc_a_hl_ */
case 0x9E:
	*a = subtract_and_test(*a, read_byte(*hl), Tst(carryFlag)? 1: 0);
	break;

/* sbc_aa could be done more efficiently */
/* sbc_aa */
case 0x9F:
	*a = subtract_and_test(*a, *a, Tst(carryFlag)? 1: 0);
	break;


/* Bitwise AND... */

/* and_b */
case 0xA0:
	*a &= *b;
	do_and_flags(*a);
	break;

/* and_c */
case 0xA1:
	*a &= *c;
	do_and_flags(*a);
	break;

/* and_d */
case 0xA2:
	*a &= *d;
	do_and_flags(*a);
	break;

/* and_e */
case 0xA3:
	*a &= *e;
	do_and_flags(*a);
	break;

/* and_h */
case 0xA4:
	*a &= *h;
	do_and_flags(*a);
	break;

/* and_l */
case 0xA5:
	*a &= *l;
	do_and_flags(*a);
	break;

/* and__hl_ */
case 0xA6:
	*a &= read_byte(*hl);
	do_and_flags(*a);
	break;

/* Don't need to do the AND operation here */
/* and_a */
case 0xA7:
	do_and_flags(*a);
	break;


/* Bitwise XOR... */
/* xor_b */
case 0xA8:
	*a ^= *b;
	do_xor_flags(*a);
	break;

/* xor_c */
case 0xA9:
	*a ^= *c;
	do_xor_flags(*a);
	break;

/* xor_d */
case 0xAA:
	*a ^= *d;
	do_xor_flags(*a);
	break;

/* xor_e */
case 0xAB:
	*a ^= *e;
	do_xor_flags(*a);
	break;

/* xor_h */
case 0xAC:
	*a ^= *h;
	do_xor_flags(*a);
	break;

/* xor_l */
case 0xAD:
	*a ^= *l;
	do_xor_flags(*a);
	break;

/* xor__hl_ */
case 0xAE:
	*a ^= read_byte(*hl);
	do_xor_flags(*a);
	break;

/* NOTE: xor_a is a special case */
/* xor_a */
case 0xAF:
	*a = 0;
	Clr(hcarryFlag); Clr(carryFlag); Clr(addsubFlag);
	Set(zeroFlag); Clr(signFlag); Set(par_overFlag);
	break;


/* And bitwise OR... */
/* or_b */
case 0xB0:
	*a |= *b;
	do_or_flags(*a);
	break;

/* or_c */
case 0xB1:
	*a |= *c;
	do_or_flags(*a);
	break;

/* or_d */
case 0xB2:
	*a |= *d;
	do_or_flags(*a);
	break;

/* or_e */
case 0xB3:
	*a |= *e;
	do_or_flags(*a);
	break;

/* or_h */
case 0xB4:
	*a |= *h;
	do_or_flags(*a);
	break;

/* or_l */
case 0xB5:
	*a |= *l;
	do_or_flags(*a);
	break;

/* or__hl_ */
case 0xB6:
	*a |= read_byte(*hl);
	do_or_flags(*a);
	break;

/* don't need to do the OR operation here */
/* or_a */
case 0xB7:
	do_or_flags(*a);
	break;


/* Register/register compare... */
/* cp_b */
case 0xB8:
	compare_and_test(*b);
	break;

/* cp_c */
case 0xB9:
	compare_and_test(*c);
	break;

/* cp_d */
case 0xBA:
	compare_and_test(*d);
	break;

/* cp_e */
case 0xBB:
	compare_and_test(*e);
	break;

/* cp_h */
case 0xBC:
	compare_and_test(*h);
	break;

/* cp_l */
case 0xBD:
	compare_and_test(*l);
	break;

/* cp__hl_ */
case 0xBE:
	compare_and_test(read_byte(*hl));
	break;

/* NOTE: cp_a is a special case */
/* cp_a */
case 0xBF:
	Set(zeroFlag); Clr(carryFlag); Clr(par_overFlag);
	Clr(signFlag); Set(addsubFlag);
	break;


/* ret_nz */
case 0xC0:
	if (!Tst(zeroFlag))
		*pc = PopValue;
	break;

/* pop_bc */
case 0xC1:
	*bc = PopValue;
	break;

/* jp_nzNN */
case 0xC2:
	if (!Tst(zeroFlag))
		*pc = GetNext2Ops;
	else
		*pc += 2;
	break;

/* jp_NN */
case 0xC3:
	*pc = GetNext2Ops;
	break;

/* call_nzNN */
case 0xC4:
	if (!Tst(zeroFlag)) {
		work16 = GetNext2Ops;
		PushValue(*pc);
		*pc = work16;
	} else
		*pc += 2;
	break;

/* push_bc */
case 0xC5:
	PushValue(*bc);
	break;

/* add_aN */
case 0xC6:
	work8 = GetNextOp;
	*a = add_and_test(*a, work8, 0);
	break;

/* rst_0x0 */
case 0xC7:
	PushValue(*pc);
	*pc = 0x0000;
	break;

/* ret_z */
case 0xC8:
	if (Tst(zeroFlag))
		*pc = PopValue;
	break;

/* ret */
case 0xC9:
	*pc = PopValue;
	break;

/* jp_zNN */
case 0xCA:
	if (Tst(zeroFlag))
		*pc = GetNext2Ops;
	else
		*pc += 2;
	break;

/* EXTEND_203 */
case 0xCB:
	work8 = GetNextOp;
do_cb_op:
	switch (work8) {
#		include "cbops.c"
	}
	break;

/* call_zNN */
case 0xCC:
	if (Tst(zeroFlag)) {
		work16 = GetNext2Ops;
		PushValue(*pc);
		*pc = work16;
	} else
		*pc += 2;
	break;

/* call_NN */
case 0xCD:
	work16 = GetNext2Ops;
	PushValue(*pc);
	*pc = work16;
	break;

/* adc_aN */
case 0xCE:
	work8 = GetNextOp;
	*a = add_and_test(*a, work8, Tst(carryFlag) ? 1 : 0);
	break;

/* rst_0x8 */
case 0xCF:
	PushValue(*pc);
	*pc = 0x0008;
	break;

/* ret_nc */
case 0xD0:
	if (!Tst(carryFlag))
		*pc = PopValue;
	break;

/* pop_de */
case 0xD1:
	*de = PopValue;
	break;

/* jp_ncNN */
case 0xD2:
	if (!Tst(carryFlag))
		*pc = GetNext2Ops;
	else
		*pc += 2;
	break;

/* out__N_a */
case 0xD3:
	out_byte((*a << 8) + GetNextOp, *a);
	break;

/* call_ncNN */
case 0xD4:
	if (!Tst(carryFlag)) {
		work16 = GetNext2Ops;
		PushValue(*pc);
		*pc = work16;
	} else
		*pc += 2;
	break;

/* push_de */
case 0xD5:
	PushValue(*de);
	break;

/* sub_N */
case 0xD6:
	work8 = GetNextOp;
	*a = subtract_and_test(*a, work8, 0);
	break;

/* rst_0x10 */
case 0xD7:
	PushValue(*pc);
	*pc = 0x0010;
	break;

/* ret_c */
case 0xD8:
	if (Tst(carryFlag))
		*pc = PopValue;
	break;

/* exx */
case 0xD9:
	work16 = theProcessor.bc_pair;
	theProcessor.bc_pair = theProcessor.bc_alt;
	theProcessor.bc_alt = work16;
	work16 = theProcessor.de_pair;
	theProcessor.de_pair = theProcessor.de_alt;
	theProcessor.de_alt = work16;
	work16 = theProcessor.hl_pair;
	theProcessor.hl_pair = theProcessor.hl_alt;
	theProcessor.hl_alt = work16;
	break;

/* jp_cNN */
case 0xDA:
	if (Tst(carryFlag))
		*pc = GetNext2Ops;
	else
		*pc += 2;
	break;

/* in a,N */
case 0xDB:
	*a = in_byte((*a << 8) + GetNextOp);
	break;

/* call_cNN */
case 0xDC:
	if (Tst(carryFlag)) {
		work16 = GetNext2Ops;
		PushValue(*pc);
		*pc = work16;
	} else
		*pc += 2;
	break;

/* opcode 0xdd - ix register handling */
case 0xDD:
	ixiy_op = 1;
	safe_hl = hl; hl = ix;	/* hl now points at the ix register */
	if (read_byte(*pc) == 0xCB) { /* DD CB ops. always have displacements */
		(*pc)++;
		disp  = GetNextOp;
		work8 = GetNextOp;
		*hl += disp;
		goto do_cb_op;
	} else {
		 work8 = GetNextOp;
		 if (ixiy_tbl[work8]) {
			disp = GetNextOp;
		 } else {
			disp = 0;
#ifdef LITTLE_ENDIAN
			h = (uns8 *)hl + 1; l = (uns8 *)hl;
#else
			h = (uns8 *)hl; l = (uns8 *)hl + 1;
#endif
		 }
		 *hl += disp;
		 op = work8;
		 goto restart_switch;
	}
	break;

/* sbc_aN */
case 0xDE:
	work8 = GetNextOp;
	*a = subtract_and_test(*a, work8, Tst(carryFlag) ? 1 : 0);
	break;

/* rst_0x18 */
case 0xDF:
	PushValue(*pc);
	*pc = 0x0018;
	break;

/* ret_po */
case 0xE0:
	if (!Tst(par_overFlag))
		*pc = PopValue;
	break;

/* pop_hl */
case 0xE1:
	*hl = PopValue;
	break;

/* jp_poNN */
case 0xE2:
	if (!Tst(par_overFlag))
		*pc = GetNext2Ops;
	else
		*pc += 2;
	break;

/* ex (sp),hl */
case 0xE3:
	work16 = read_word(*sp);
	write_word(*sp,*hl);
	*hl = work16;
	break;

/* call_poNN */
case 0xE4:
	if (!Tst(par_overFlag)) {
		work16 = GetNext2Ops;
		PushValue(*pc);
		*pc = work16;
	} else
		*pc += 2;
	break;

/* push_hl */
case 0xE5:
	PushValue(*hl);
	break;

/* and_N */
case 0xE6:
	work8 = GetNextOp;
	*a &= work8;
	do_and_flags(*a);
	break;

/* rst_0x20 */
case 0xE7:
	PushValue(*pc);
	*pc = 0x0020;
	break;

/* ret_pe */
case 0xE8:
	if (Tst(par_overFlag))
		*pc = PopValue;
	break;

/* jp__hl_ */
case 0xE9:
	*pc = *hl;
	break;

/* jp_peNN */
case 0xEA:
	if (Tst(par_overFlag))
		*pc = GetNext2Ops;
	else
		*pc += 2;
	break;

/* ex_dehl */
case 0xEB:
	work16 = *de; *de = *hl; *hl = work16;
	break;

/* call_peNN */
case 0xEC:
	if (Tst(par_overFlag)) {
		work16 = GetNext2Ops;
		PushValue(*pc);
		*pc = work16;
	} else
		*pc += 2;
	break;

/* EXTEND_237 */
case 0xED:
	work8 = GetNextOp;
	switch (work8) {
#		include "edops.c"
	}
	break;

/* xor_N */
case 0xEE:
	work8 = GetNextOp;
	*a ^= work8;
	do_xor_flags(*a);
	break;

/* rst_0x28 */
case 0xEF:
	PushValue(*pc);
	*pc = 0x0028;
	break;

/* ret_p */
case 0xF0:
	if (!Tst(signFlag))
		*pc = PopValue;
	break;

/* pop_af */
case 0xF1:
	*af = PopValue;
	RetrieveFlags();
	break;

/* jp_pNN */
case 0xF2:
	if (!Tst(signFlag)) {
		*pc = GetNext2Ops;
	} else
		*pc += 2;
	break;

/* di */
case 0xF3:
	theProcessor.iff1 = 0;
	theProcessor.iff2 = 0;
	break;

/* call_pNN */
case 0xF4:
	if (!Tst(signFlag)) {
		work16 = GetNext2Ops;
		PushValue(*pc);
		*pc = work16;
	} else
		*pc += 2;
	break;

/* push_af */
case 0xF5:
	StoreFlags();
	PushValue(*af);
	break;

/* or_N */
case 0xF6:
	work8 = GetNextOp;
	*a |= work8;
	do_or_flags(*a);
	break;

/* rst_0x30 */
case 0xF7:
	PushValue(*pc);
	*pc = 0x0030;
	break;

/* ret_m */
case 0xF8:
	if (Tst(signFlag))
		*pc = PopValue;
	break;

/* ld_sphl */
case 0xF9:
	*sp = *hl;
	break;

/* jp_mNN */
case 0xFA:
	if (Tst(signFlag)) {
		*pc = GetNext2Ops;
	} else
		*pc += 2;
	break;

/* ei */
case 0xFB:
	theProcessor.iff1 = 1;
	theProcessor.iff2 = 1;
	break;

/* call_mNN */
case 0xFC:
	if (Tst(signFlag)) {
		work16 = GetNext2Ops;
		PushValue(*pc);
		*pc = work16;
	} else
		*pc += 2;
	break;

/* opcode 0xfd - iy register handling */
case 0xFD:
	ixiy_op = 1;
	safe_hl = hl; hl = iy;	/* hl now points at the iy register */
	if (read_byte(*pc) == 0xCB) { /* FD CB ops. always have displacements */
		(*pc)++;
		disp  = GetNextOp;
		work8 = GetNextOp;
		*hl += disp;
		goto do_cb_op;
	} else {
		 work8 = GetNextOp;
		 if (ixiy_tbl[work8]) {
			disp = GetNextOp;
		 } else {
			disp = 0;
#ifdef LITTLE_ENDIAN
			h = (uns8 *)hl + 1; l = (uns8 *)hl;
#else
			h = (uns8 *)hl; l = (uns8 *)hl + 1;
#endif
		 }
		 *hl += disp;
		 op = work8;
		 goto restart_switch;
	}
	break;

/* cp_N */
case 0xFE:
	work8  = GetNextOp;
	compare_and_test(work8);
	break;

/* rst_0x38 */
case 0xFF:
	PushValue(*pc);
	*pc = 0x0038;
	break;
